#include "dyndlg.h"

//
// wptr is pointer to dialog box that contains this class
// homespot is ID of the frame that is correctly positioned
// hideatstart tells the class to send SW_HIDE to all inactive controls on
//   startup--but it's faster to unset VISIBLE attribute in dialog editor
// firstactive is the number of the set to be initially shown
// maxsets means no more than that many can be saved
// typical call is--
//   TMyDialog::TMyDialog(...) : TDialog(...), mydyncontrols(this,HomeFrameID){}
//     where mydyncontrols is a data member of TMyDialog
DynamicDialogControls::DynamicDialogControls(BOOL hideatstart, int firstactive,
		int maxsets) : HideAtStart(hideatstart), Active(firstactive),
	NumberOfSets(0)
{
	Starts = new int[maxsets] ;
	Ends = new int[maxsets] ;
}

DynamicDialogControls::~DynamicDialogControls()
{
	delete []Starts ;
	delete []Ends ;
}

void DynamicDialogControls::Setup(int homespot, HWND hdlgwindow)
{
	HomeSpot = homespot ;
	HDlgWindow = hdlgwindow ;
}

// add a set of sequentially numbered controls to dynamic box--they will be
// moved to the same relative position in the HomeSpot frame as they were in the
// refbox frame
// returns the set number

int DynamicDialogControls::AddSet(int firstitem, int lastitem, int refbox)
{
	Starts[NumberOfSets] = firstitem ;
	Ends[NumberOfSets] = lastitem ;
	if (refbox != HomeSpot)
		MoveSet(NumberOfSets, refbox) ;
	if (HideAtStart && NumberOfSets != Active)
		ShowSet(NumberOfSets, SW_HIDE) ;
	return NumberOfSets++ ;
}

void DynamicDialogControls::GetDlgCtlRect(int nID, LPRECT lprc)
{
	::GetWindowRect(::GetDlgItem(HDlgWindow,nID),lprc) ;
	::MapWindowPoints(NULL, HDlgWindow, (POINT FAR*) lprc, 2) ;
}

// activate a set--returns the number of the prior active set
int DynamicDialogControls::Activate(int which)
{
	int lastactive = Active ;
	if (Active==which)
		return Active ;
	ShowSet(Active,SW_HIDE) ;
	ShowSet(which, SW_SHOW) ;
	Active = which ;
	return lastactive ;
}

void DynamicDialogControls::MoveSet(int setnumber, int refbox)
{
	RECT rcSrc, rcDest, rcCtl ;
	int nDeltaX, nDeltaY ;
	HWND hwndCtl ;

	PRECONDITIONX(::GetDlgItem(HDlgWindow,refbox), "refbox in call to MoveSet is"
		"invalid") ;
	PRECONDITIONX(::GetDlgItem(HDlgWindow,HomeSpot), "HomeSpot in call to MoveSet is"
		"invalid") ;
	GetDlgCtlRect(refbox, &rcSrc) ;
	GetDlgCtlRect(HomeSpot, &rcDest) ;

	nDeltaX = rcDest.left - rcSrc.left ;
	nDeltaY = rcDest.top - rcSrc.top ;

	for (int i = Starts[setnumber] ; i <= Ends[setnumber] ; i++)
	{
		hwndCtl = ::GetDlgItem(HDlgWindow,i) ;
		if (hwndCtl)
		{
			GetDlgCtlRect(i, &rcCtl) ;
			::SetWindowPos(hwndCtl, NULL, rcCtl.left + nDeltaX,
				rcCtl.top + nDeltaY, 0, 0,
				SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE) ;
		}
	}
}


void DynamicDialogControls::ShowSet(int setnumber, int cmdShow)
{
	HWND hwndCtl ;
	for (int i = Starts[setnumber] ; i <= Ends[setnumber] ; i++)
	{
		hwndCtl = ::GetDlgItem(HDlgWindow,i) ;
		if (hwndCtl)
			::ShowWindow(hwndCtl, cmdShow) ;
	}
}