#ifndef __DYNDLG_H
#define __DYNDLG_H
#include <owl\owlpch.h>

// Class DynamicDialogControls by Rory Jaffe
// the code for functions: GetDlgCtlRect and MoveSet are adapted (with changes)
//  from the code by Atif Aziz in the article: "Simplify and Enhance Your
//  Application's User Interface with Dynamic Dialog Boxes" in MSJ vol. 9 no. 3,
//	 Mar, 1994.  Both functions are changed to to replace a few function calls
//  which appear to be either erroneous or are calls to functions he has defined
//  but not printed in the article. There are at least two things you can do to
//  create dangerous situations with this code: 1) use non-existent ID's for
//  homespot or refbox, and 2) add more than maxsets sets. I've added debug
//  macros for number 1--it should be unnecessary for your finished code.


// INTERFACE:
//  constructor--rarely need to alter default arguments--make hideatstart TRUE
//    if you did not remove visible attribute from the controls which are not
//    to be initially visible--make firstactive nonzero if you want a different
//    set (than 0) initially active--increase maxsets if you are crazy enough
//    to have more than 20 "looks" for the dialog box
//  Setup--this function is called in SetupWindow method of dialog box--first
//    argument is resource ID of the frame that is correctly positioned in the
//    dialog box, as a reference for moving the sets of controls--second
//    argument should be the HWND of the dialog box owning the controls, which
//    is almost always HWindow
//  AddSet--adds in sequence each set of dynamic controls (sets numbered from 0) and
//    returns the sequence number for the added set--each set consists of a
//    group of sequentially numbered controls (gaps are OK but wasteful) and a
//    refence frame.  The controls are placed in the reference frame so that
//    they are in the same relation to their reference frame as you wish them
//    to be with the correctly positioned frame (set in Setup method).
//		The AddSet method should also be called in the dialog class's SetupWindow
//    method.
//  Activate--argument is the set number of the group to activate (show).  This
//    function returns the set number of the previously active group.
class DynamicDialogControls
{
public:
	DynamicDialogControls(BOOL hideatstart=FALSE,
		int firstactive=0, int maxsets=20) ;
	~DynamicDialogControls() ;
	void Setup(int homespot, HWND hdlgwindow) ;
	int AddSet(int firstitem, int lastitem, int refbox) ;
	int Activate(int which) ;
private:
	void ShowSet(int setnumber, int cmdshow) ;
	void MoveSet(int setnumber, int refbox) ;
	void GetDlgCtlRect(int nID, LPRECT lprc) ;
	HWND HDlgWindow ;
	int HomeSpot ;
	int NumberOfSets ;
	int Active ;
	int* Starts ;
	int* Ends ;
	BOOL HideAtStart ;
} ;


#endif
